package javax.management;

import java.util.Arrays;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import com.sun.jmx.mbeanserver.MXBeanProxy;
import java.lang.ref.WeakReference;
import java.util.WeakHashMap;
import java.lang.reflect.InvocationHandler;

public class MBeanServerInvocationHandler implements InvocationHandler
{
    private static final WeakHashMap<Class<?>, WeakReference<MXBeanProxy>> mxbeanProxies;
    private final MBeanServerConnection connection;
    private final ObjectName objectName;
    private final boolean isMXBean;
    
    public MBeanServerInvocationHandler(MBeanServerConnection mBeanServerConnection, ObjectName objectName) {
        this(mBeanServerConnection, objectName, false);
    }
    
    public MBeanServerInvocationHandler(MBeanServerConnection mBeanServerConnection, ObjectName objectName, boolean isMXBean) {
        if (mBeanServerConnection == null) {
            throw new IllegalArgumentException("Null connection");
        }
        if (Proxy.isProxyClass(mBeanServerConnection.getClass()) && MBeanServerInvocationHandler.class.isAssignableFrom(Proxy.getInvocationHandler(mBeanServerConnection).getClass())) {
            throw new IllegalArgumentException("Wrapping MBeanServerInvocationHandler");
        }
        if (objectName == null) {
            throw new IllegalArgumentException("Null object name");
        }
        this.connection = mBeanServerConnection;
        this.objectName = objectName;
        this.isMXBean = isMXBean;
        return;
    }
    
    public MBeanServerConnection getMBeanServerConnection() {
        return this.connection;
    }
    
    public ObjectName getObjectName() {
        return this.objectName;
    }
    
    public boolean isMXBean() {
        return this.isMXBean;
    }
    
    public static <T> T newProxyInstance(MBeanServerConnection mBeanServerConnection, ObjectName objectName, Class<T> clazz, boolean b) {
        return JMX.newMBeanProxy(mBeanServerConnection, objectName, clazz, b);
    }
    
    @Override
    public Object invoke(Object o, Method method, Object[] args) throws Throwable {
        Class<Object> declaringClass = method.getDeclaringClass();
        if (declaringClass.equals(NotificationBroadcaster.class) || declaringClass.equals(NotificationEmitter.class)) {
            return this.invokeBroadcasterMethod(o, method, args);
        }
        if (this.shouldDoLocally(o, method)) {
            return this.doLocally(o, method, args);
        }
        try {
            if (this.isMXBean()) {
                return findMXBeanProxy(declaringClass).invoke(this.connection, this.objectName, method, args);
            }
            String name = method.getName();
            Class[] parameterTypes = method.getParameterTypes();
            Class<Object> returnType = method.getReturnType();
            int n = (args == null) ? 0 : args.length;
            if (name.startsWith("get") && name.length() > 3 && n == 0 && !returnType.equals(Void.TYPE)) {
                return this.connection.getAttribute(this.objectName, name.substring(3));
            }
            if (name.startsWith("is") && name.length() > 2 && n == 0 && (returnType.equals(Boolean.TYPE) || returnType.equals(Boolean.class))) {
                return this.connection.getAttribute(this.objectName, name.substring(2));
            }
            if (name.startsWith("set") && name.length() > 3 && n == 1 && returnType.equals(Void.TYPE)) {
                this.connection.setAttribute(this.objectName, new Attribute(name.substring(3), args[0]));
                return null;
            }
            String[] array = new String[parameterTypes.length];
            for (int i = 0; i < parameterTypes.length; ++i) {
                array[i] = parameterTypes[i].getName();
            }
            return this.connection.invoke(this.objectName, name, args, array);
        }
        catch (MBeanException ex) {
            throw ex.getTargetException();
        }
        catch (RuntimeMBeanException ex2) {
            throw ex2.getTargetException();
        }
        catch (RuntimeErrorException ex3) {
            throw ex3.getTargetError();
        }
    }
    
    private static MXBeanProxy findMXBeanProxy(Class<?> key) {
        synchronized (MBeanServerInvocationHandler.mxbeanProxies) {
            WeakReference<Object> weakReference = (WeakReference)MBeanServerInvocationHandler.mxbeanProxies.get(key);
            MXBeanProxy referent = (weakReference == null) ? null : ((MXBeanProxy)weakReference.get());
            if (referent == null) {
                try {
                    referent = new MXBeanProxy(key);
                }
                catch (IllegalArgumentException ex) {
                    IllegalArgumentException ex2 = new IllegalArgumentException("Cannot make MXBean proxy for " + key.getName() + ": " + ex.getMessage(), ex.getCause());
                    ex2.setStackTrace(ex.getStackTrace());
                    throw ex2;
                }
                MBeanServerInvocationHandler.mxbeanProxies.put(key, new WeakReference(referent));
            }
            return referent;
        }
    }
    
    private Object invokeBroadcasterMethod(Object o, Method method, Object[] array) throws Exception {
        String name = method.getName();
        int n = (array == null) ? 0 : array.length;
        if (name.equals("addNotificationListener")) {
            if (n != 3) {
                throw new IllegalArgumentException("Bad arg count to addNotificationListener: " + n);
            }
            this.connection.addNotificationListener(this.objectName, (NotificationListener)array[0], (NotificationFilter)array[1], array[2]);
            return null;
        }
        else if (name.equals("removeNotificationListener")) {
            NotificationListener notificationListener = (NotificationListener)array[0];
            switch (n) {
                case 1: {
                    this.connection.removeNotificationListener(this.objectName, notificationListener);
                    return null;
                }
                case 3: {
                    this.connection.removeNotificationListener(this.objectName, notificationListener, (NotificationFilter)array[1], array[2]);
                    return null;
                }
                default: {
                    throw new IllegalArgumentException("Bad arg count to removeNotificationListener: " + n);
                }
            }
        }
        else if (name.equals("getNotificationInfo")) {
            if (array != null) {
                throw new IllegalArgumentException("getNotificationInfo has args");
            }
            return this.connection.getMBeanInfo(this.objectName).getNotifications();
        }
        else {
            throw new IllegalArgumentException("Bad method name: " + name);
        }
    }
    
    private boolean shouldDoLocally(Object o, Method method) {
        String name = method.getName();
        return ((name.equals("hashCode") || name.equals("toString")) && method.getParameterTypes().length == 0 && isLocal(o, method)) || (name.equals("equals") && Arrays.equals(method.getParameterTypes(), new Class[] { Object.class }) && isLocal(o, method)) || (name.equals("finalize") && method.getParameterTypes().length == 0);
    }
    
    private Object doLocally(Object o, Method method, Object[] array) {
        String name = method.getName();
        if (name.equals("equals")) {
            if (this == array[0]) {
                return Boolean.valueOf(true);
            }
            if (!(array[0] instanceof Proxy)) {
                return Boolean.valueOf(false);
            }
            InvocationHandler invocationHandler = Proxy.getInvocationHandler(array[0]);
            if (invocationHandler == null || !(invocationHandler instanceof MBeanServerInvocationHandler)) {
                return Boolean.valueOf(false);
            }
            MBeanServerInvocationHandler mBeanServerInvocationHandler = (MBeanServerInvocationHandler)invocationHandler;
            return Boolean.valueOf(this.connection.equals(mBeanServerInvocationHandler.connection) && this.objectName.equals(mBeanServerInvocationHandler.objectName) && o.getClass().equals(array[0].getClass()));
        }
        else {
            if (name.equals("toString")) {
                return (this.isMXBean() ? "MX" : "M") + "BeanProxy(" + this.connection + "[" + this.objectName + "])";
            }
            if (name.equals("hashCode")) {
                return Integer.valueOf(this.objectName.hashCode() + this.connection.hashCode());
            }
            if (name.equals("finalize")) {
                return null;
            }
            throw new RuntimeException("Unexpected method name: " + name);
        }
    }
    
    private static boolean isLocal(Object o, Method method) {
        Class[] interfaces = o.getClass().getInterfaces();
        if (interfaces == null) {
            return true;
        }
        String name = method.getName();
        Class[] parameterTypes = method.getParameterTypes();
        int length = interfaces.length;
        int i = 0;
        while (i < length) {
            Class<Object> clazz = interfaces[i];
            try {
                clazz.getMethod(name, parameterTypes);
                return false;
            }
            catch (NoSuchMethodException ex) {
                ++i;
            }
        }
        return true;
    }
    
    static {
        mxbeanProxies = new WeakHashMap();
    }
}
